#import <UIKit/UIKit.h>
#import <OpenGLES/EAGL.h>
#import <OpenGLES/ES1/gl.h>
#import <OpenGLES/ES1/glext.h>

/*
 This class wraps the CAEAGLLayer from CoreAnimation into a convenient UIView subclass.
 The view content is basically an EAGL surface you render your OpenGL scene into.
 Note that setting the view non-opaque will only work if the EAGL surface has an alpha channel.
 */
@interface EAGLView : UIView {
    
@private
    /* The pixel dimensions of the backbuffer */
    GLint backingWidth;
    GLint backingHeight;
    
    EAGLContext *context;
    
    /* OpenGL names for the renderbuffer and framebuffers used to render to this view */
    GLuint viewRenderbuffer, viewFramebuffer;
    
    /* OpenGL name for the depth buffer that is attached to viewFramebuffer, if it exists (0 if it does not exist) */
    GLuint depthRenderbuffer;
    
    NSTimer *animationTimer;
    NSTimeInterval animationInterval;
	
	// Texture dimensions must be a power of 2
	GLuint texture;
	int textureWidth, textureHeight;
	GLshort textureVertices[8];
	GLfloat textureCoords[8];
	uint8_t *texturePixels;
}

@property NSTimeInterval animationInterval;

/* Must be called AFTER the camera has been loaded by the Image Picker Controller in MainWindow.xib, typically in applicationDidFinishLaunching */
- (void)installCameraCallbackHook;

- (void)startAnimation;
- (void)stopAnimation;
- (void)drawView;

@end
